summaryrefslogtreecommitdiff
path: root/src/mongo
diff options
context:
space:
mode:
authorJason Rassi <rassi@10gen.com>2015-08-11 10:55:17 -0400
committerJason Rassi <rassi@10gen.com>2015-08-11 11:01:00 -0400
commit8e2990da71c1498d2a979ef47c07c2bfc2fa4f97 (patch)
tree8dec46fd85e96504dec09315fbe596b92a6e3ca8 /src/mongo
parentff9624152b6b9d017d65cf5055dc5f3a9907f062 (diff)
downloadmongo-8e2990da71c1498d2a979ef47c07c2bfc2fa4f97.tar.gz
SERVER-18774 Add ClockSource interface
Diffstat (limited to 'src/mongo')
-rw-r--r--src/mongo/SConscript1
-rw-r--r--src/mongo/db/service_context.cpp8
-rw-r--r--src/mongo/db/service_context.h11
-rw-r--r--src/mongo/db/service_context_d.cpp2
-rw-r--r--src/mongo/s/server.cpp2
-rw-r--r--src/mongo/util/SConscript7
-rw-r--r--src/mongo/util/clock_source.h48
-rw-r--r--src/mongo/util/clock_source_mock.cpp48
-rw-r--r--src/mongo/util/clock_source_mock.h62
-rw-r--r--src/mongo/util/system_clock_source.cpp47
-rw-r--r--src/mongo/util/system_clock_source.h48
11 files changed, 280 insertions, 4 deletions
diff --git a/src/mongo/SConscript b/src/mongo/SConscript
index b67a9630633..bab71bd6ac8 100644
--- a/src/mongo/SConscript
+++ b/src/mongo/SConscript
@@ -102,6 +102,7 @@ baseSource=[
'util/stacktrace.cpp',
'util/static_observer.cpp',
'util/stringutils.cpp',
+ 'util/system_clock_source.cpp',
'util/system_tick_source.cpp',
'util/text.cpp',
'util/time_support.cpp',
diff --git a/src/mongo/db/service_context.cpp b/src/mongo/db/service_context.cpp
index 8dfa7bfdc98..7fe0e237bdc 100644
--- a/src/mongo/db/service_context.cpp
+++ b/src/mongo/db/service_context.cpp
@@ -147,10 +147,18 @@ TickSource* ServiceContext::getTickSource() const {
return _tickSource.get();
}
+ClockSource* ServiceContext::getClockSource() const {
+ return _clockSource.get();
+}
+
void ServiceContext::setTickSource(std::unique_ptr<TickSource> newSource) {
_tickSource = std::move(newSource);
}
+void ServiceContext::setClockSource(std::unique_ptr<ClockSource> newSource) {
+ _clockSource = std::move(newSource);
+}
+
void ServiceContext::ClientDeleter::operator()(Client* client) const {
ServiceContext* const service = client->getServiceContext();
{
diff --git a/src/mongo/db/service_context_d.cpp b/src/mongo/db/service_context_d.cpp
index cef92203f59..af17b210db9 100644
--- a/src/mongo/db/service_context_d.cpp
+++ b/src/mongo/db/service_context_d.cpp
@@ -50,6 +50,7 @@
#include "mongo/util/map_util.h"
#include "mongo/util/mongoutils/str.h"
#include "mongo/util/scopeguard.h"
+#include "mongo/util/system_clock_source.h"
#include "mongo/util/system_tick_source.h"
namespace mongo {
@@ -57,6 +58,7 @@ namespace mongo {
MONGO_INITIALIZER(SetGlobalEnvironment)(InitializerContext* context) {
setGlobalServiceContext(stdx::make_unique<ServiceContextMongoD>());
getGlobalServiceContext()->setTickSource(stdx::make_unique<SystemTickSource>());
+ getGlobalServiceContext()->setClockSource(stdx::make_unique<SystemClockSource>());
return Status::OK();
}
diff --git a/src/mongo/s/server.cpp b/src/mongo/s/server.cpp
index a2c7cec3ff6..d74f4823aa7 100644
--- a/src/mongo/s/server.cpp
+++ b/src/mongo/s/server.cpp
@@ -85,6 +85,7 @@
#include "mongo/util/stacktrace.h"
#include "mongo/util/static_observer.h"
#include "mongo/util/stringutils.h"
+#include "mongo/util/system_clock_source.h"
#include "mongo/util/system_tick_source.h"
#include "mongo/util/text.h"
#include "mongo/util/version.h"
@@ -360,6 +361,7 @@ MONGO_INITIALIZER(CreateAuthorizationExternalStateFactory)(InitializerContext* c
MONGO_INITIALIZER(SetGlobalEnvironment)(InitializerContext* context) {
setGlobalServiceContext(stdx::make_unique<ServiceContextNoop>());
getGlobalServiceContext()->setTickSource(stdx::make_unique<SystemTickSource>());
+ getGlobalServiceContext()->setClockSource(stdx::make_unique<SystemClockSource>());
return Status::OK();
}
diff --git a/src/mongo/util/SConscript b/src/mongo/util/SConscript
index bb0528e6077..b932c45a325 100644
--- a/src/mongo/util/SConscript
+++ b/src/mongo/util/SConscript
@@ -124,6 +124,13 @@ env.Library(
],
)
+env.Library(
+ target='clock_source_mock',
+ source=[
+ 'clock_source_mock.cpp',
+ ],
+)
+
env.CppUnitTest(
target='text_test',
source=[
diff --git a/src/mongo/util/clock_source.h b/src/mongo/util/clock_source.h
new file mode 100644
index 00000000000..6ba47e6fce3
--- /dev/null
+++ b/src/mongo/util/clock_source.h
@@ -0,0 +1,48 @@
+/**
+ * Copyright (C) 2015 MongoDB Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * As a special exception, the copyright holders give permission to link the
+ * code of portions of this program with the OpenSSL library under certain
+ * conditions as described in each individual source file and distribute
+ * linked combinations including the program with the OpenSSL library. You
+ * must comply with the GNU Affero General Public License in all respects for
+ * all of the code used other than as permitted herein. If you modify file(s)
+ * with this exception, you may extend this exception to your version of the
+ * file(s), but you are not obligated to do so. If you do not wish to do so,
+ * delete this exception statement from your version. If you delete this
+ * exception statement from all source files in the program, then also delete
+ * it in the license file.
+ */
+
+#pragma once
+
+namespace mongo {
+
+class Date_t;
+
+/**
+ * An interface for getting the current wall clock time.
+ */
+class ClockSource {
+public:
+ virtual ~ClockSource() = default;
+
+ /**
+ * Returns the current wall clock time, as defined by this source.
+ */
+ virtual Date_t now() = 0;
+};
+
+} // namespace mongo
diff --git a/src/mongo/util/clock_source_mock.cpp b/src/mongo/util/clock_source_mock.cpp
new file mode 100644
index 00000000000..ba5c9cef2e3
--- /dev/null
+++ b/src/mongo/util/clock_source_mock.cpp
@@ -0,0 +1,48 @@
+/**
+ * Copyright (C) 2015 MongoDB Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * As a special exception, the copyright holders give permission to link the
+ * code of portions of this program with the OpenSSL library under certain
+ * conditions as described in each individual source file and distribute
+ * linked combinations including the program with the OpenSSL library. You
+ * must comply with the GNU Affero General Public License in all respects for
+ * all of the code used other than as permitted herein. If you modify file(s)
+ * with this exception, you may extend this exception to your version of the
+ * file(s), but you are not obligated to do so. If you do not wish to do so,
+ * delete this exception statement from your version. If you delete this
+ * exception statement from all source files in the program, then also delete
+ * it in the license file.
+ */
+
+#include "mongo/platform/basic.h"
+
+#include "mongo/util/clock_source_mock.h"
+
+namespace mongo {
+
+ClockSourceMock::ClockSourceMock(Date_t now) : _now(now) {}
+
+Date_t ClockSourceMock::now() {
+ return _now;
+}
+
+void ClockSourceMock::advance(stdx::chrono::milliseconds ms) {
+ _now += ms;
+}
+
+void ClockSourceMock::reset(Date_t newNow) {
+ _now = newNow;
+}
+} // namespace mongo
diff --git a/src/mongo/util/clock_source_mock.h b/src/mongo/util/clock_source_mock.h
new file mode 100644
index 00000000000..2f53abc23b6
--- /dev/null
+++ b/src/mongo/util/clock_source_mock.h
@@ -0,0 +1,62 @@
+/**
+ * Copyright (C) 2015 MongoDB Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * As a special exception, the copyright holders give permission to link the
+ * code of portions of this program with the OpenSSL library under certain
+ * conditions as described in each individual source file and distribute
+ * linked combinations including the program with the OpenSSL library. You
+ * must comply with the GNU Affero General Public License in all respects for
+ * all of the code used other than as permitted herein. If you modify file(s)
+ * with this exception, you may extend this exception to your version of the
+ * file(s), but you are not obligated to do so. If you do not wish to do so,
+ * delete this exception statement from your version. If you delete this
+ * exception statement from all source files in the program, then also delete
+ * it in the license file.
+ */
+
+#pragma once
+
+#include "mongo/util/clock_source.h"
+#include "mongo/util/time_support.h"
+
+namespace mongo {
+
+/**
+ * Mock clock source that returns a fixed time until explicitly advanced.
+ */
+class ClockSourceMock final : public ClockSource {
+public:
+ /**
+ * Constructs a ClockSourceMock with the current time initialized to the given value.
+ */
+ explicit ClockSourceMock(Date_t now);
+
+ Date_t now() final;
+
+ /**
+ * Advances the current time by the given value.
+ */
+ void advance(stdx::chrono::milliseconds ms);
+
+ /**
+ * Resets the current time to the given value.
+ */
+ void reset(Date_t newNow);
+
+private:
+ Date_t _now;
+};
+
+} // namespace mongo
diff --git a/src/mongo/util/system_clock_source.cpp b/src/mongo/util/system_clock_source.cpp
new file mode 100644
index 00000000000..ac64e49ca62
--- /dev/null
+++ b/src/mongo/util/system_clock_source.cpp
@@ -0,0 +1,47 @@
+/**
+ * Copyright (C) 2015 MongoDB Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * As a special exception, the copyright holders give permission to link the
+ * code of portions of this program with the OpenSSL library under certain
+ * conditions as described in each individual source file and distribute
+ * linked combinations including the program with the OpenSSL library. You
+ * must comply with the GNU Affero General Public License in all respects for
+ * all of the code used other than as permitted herein. If you modify file(s)
+ * with this exception, you may extend this exception to your version of the
+ * file(s), but you are not obligated to do so. If you do not wish to do so,
+ * delete this exception statement from your version. If you delete this
+ * exception statement from all source files in the program, then also delete
+ * it in the license file.
+ */
+
+#include "mongo/platform/basic.h"
+
+#include "mongo/util/system_clock_source.h"
+
+#include "mongo/stdx/memory.h"
+#include "mongo/util/time_support.h"
+
+namespace mongo {
+
+Date_t SystemClockSource::now() {
+ return Date_t::now();
+}
+
+SystemClockSource* SystemClockSource::get() {
+ static const auto globalSystemClockSource = stdx::make_unique<SystemClockSource>();
+ return globalSystemClockSource.get();
+}
+
+} // namespace mongo
diff --git a/src/mongo/util/system_clock_source.h b/src/mongo/util/system_clock_source.h
new file mode 100644
index 00000000000..69f2b661d86
--- /dev/null
+++ b/src/mongo/util/system_clock_source.h
@@ -0,0 +1,48 @@
+/**
+ * Copyright (C) 2015 MongoDB Inc.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * As a special exception, the copyright holders give permission to link the
+ * code of portions of this program with the OpenSSL library under certain
+ * conditions as described in each individual source file and distribute
+ * linked combinations including the program with the OpenSSL library. You
+ * must comply with the GNU Affero General Public License in all respects for
+ * all of the code used other than as permitted herein. If you modify file(s)
+ * with this exception, you may extend this exception to your version of the
+ * file(s), but you are not obligated to do so. If you do not wish to do so,
+ * delete this exception statement from your version. If you delete this
+ * exception statement from all source files in the program, then also delete
+ * it in the license file.
+ */
+
+#pragma once
+
+#include "mongo/util/clock_source.h"
+
+namespace mongo {
+
+/**
+ * Clock source based on the system clock.
+ */
+class SystemClockSource final : public ClockSource {
+public:
+ Date_t now() final;
+
+ /**
+ * Returns the singleton instance of SystemClockSource.
+ */
+ static SystemClockSource* get();
+};
+
+} // namespace mongo
diff --git a/src/mongo/db/service_context.h b/src/mongo/db/service_context.h
index 8f07c34d5a1..6f9aa36a22f 100644
--- a/src/mongo/db/service_context.h
+++ b/src/mongo/db/service_context.h
@@ -36,6 +36,7 @@
#include "mongo/platform/unordered_set.h"
#include "mongo/stdx/functional.h"
#include "mongo/util/decorable.h"
+#include "mongo/util/clock_source.h"
#include "mongo/util/tick_source.h"
namespace mongo {
@@ -308,16 +309,17 @@ public:
virtual OpObserver* getOpObserver() = 0;
/**
* Returns the tick source set in this context.
*/
TickSource* getTickSource() const;
+ ClockSource* getClockSource() const;
/**
* Replaces the current tick source with a new one. In other words, the old tick source
* will be destroyed. So make sure that no one is using the old tick source when
* calling this.
*/
void setTickSource(std::unique_ptr<TickSource> newSource);
+ void setClockSource(std::unique_ptr<ClockSource> newSource);
protected:
ServiceContext() = default;
@@ -341,6 +343,7 @@ private:
ClientSet _clients;
std::unique_ptr<TickSource> _tickSource;
+ std::unique_ptr<ClockSource> _clockSource;
};
/**
@@ -308,16 +309,17 @@ public:
virtual OpObserver* getOpObserver() = 0;
/**
- * Returns the tick source set in this context.
+ * Returns the tick/clock source set in this context.
*/
TickSource* getTickSource() const;
@@ -308,16 +309,17 @@ public:
/**
- * Replaces the current tick source with a new one. In other words, the old tick source
- * will be destroyed. So make sure that no one is using the old tick source when
- * calling this.
+ * Replaces the current tick/clock source with a new one. In other words, the old source will be
+ * destroyed. So make sure that no one is using the old source when calling this.
*/
void setTickSource(std::unique_ptr<TickSource> newSource);